@@ -106,10 +106,6 @@ function onMessage(request, sender, callback) {
         µm.gotoExtensionURL(request);
         break;
 
-    case 'gotoURL':
-        µm.gotoURL(request);
-        break;
-
     case 'graylistMatrixCell':
         µm.tMatrix.graylistCell(
             request.srcHostname,
@@ -226,7 +222,7 @@ var matrixSnapshot = function(pageStore, details) {
 
     let r = {
         appVersion: vAPI.app.version,
-        blockedCount: pageStore.requestStats.blocked.all,
+        blockedCount: pageStore.perLoadBlockedRequestCount,
         collapseAllDomains: µmuser.popupCollapseAllDomains,
         collapseBlacklistedDomains: µmuser.popupCollapseBlacklistedDomains,
         diff: [],
@@ -887,7 +883,7 @@ var restoreUserData = function(userData) {
 
     // If we are going to restore all, might as well wipe out clean local
     // storage
-    µm.XAL.keyvalRemoveAll(onAllRemoved);
+    vAPI.storage.clear(onAllRemoved);
 };
 
 /******************************************************************************/
@@ -896,7 +892,7 @@ var resetUserData = function() {
     var onAllRemoved = function() {
         vAPI.app.restart();
     };
-    µm.XAL.keyvalRemoveAll(onAllRemoved);
+    vAPI.storage.clear(onAllRemoved);
 };
 
 /******************************************************************************/
