@@ -196,12 +196,20 @@ var onBeforeSendHeadersHandler = function(details) {
     // to scope on unknown scheme? Etc.
     // https://github.com/gorhill/httpswitchboard/issues/191
     // https://github.com/gorhill/httpswitchboard/issues/91#issuecomment-37180275
-    let tabId = details.tabId,
-        pageStore = µm.mustPageStoreFromTabId(tabId),
-        srcHn = pageStore.pageHostname,
-        desHn = µmuri.hostnameFromURI(desURL),
-        requestType = requestTypeNormalizer[details.type] || 'other',
-        requestHeaders = details.requestHeaders;
+    const tabId = details.tabId;
+    const pageStore = µm.mustPageStoreFromTabId(tabId);
+    const desHn = µmuri.hostnameFromURI(desURL);
+    const requestType = requestTypeNormalizer[details.type] || 'other';
+    const requestHeaders = details.requestHeaders;
+
+    // https://github.com/uBlockOrigin/uMatrix-issues/issues/155
+    // https://github.com/uBlockOrigin/uMatrix-issues/issues/159
+    //   TODO: import all filtering context improvements from uBO.
+    const srcHn = tabId < 0 ||
+          details.parentFrameId < 0 ||
+          details.parentFrameId === 0 && details.type === 'sub_frame'
+        ? µmuri.hostnameFromURI(details.documentUrl) || pageStore.pageHostname
+        : pageStore.pageHostname;
 
     // https://github.com/gorhill/httpswitchboard/issues/342
     // Is this hyperlink auditing?
@@ -341,7 +349,7 @@ var onBeforeSendHeadersHandler = function(details) {
 // This fixes:
 // https://github.com/gorhill/httpswitchboard/issues/35
 
-var onHeadersReceived = function(details) {
+var onHeadersReceivedHandler = function(details) {
     // Ignore schemes other than 'http...'
     let µm = µMatrix,
         tabId = details.tabId,
@@ -502,25 +510,6 @@ var requestTypeNormalizer = {
     'xmlhttprequest': 'xhr'
 };
 
-/******************************************************************************/
-
-vAPI.net.onBeforeRequest = {
-    extra: [ 'blocking' ],
-    callback: onBeforeRequestHandler
-};
-
-vAPI.net.onBeforeSendHeaders = {
-    extra: [ 'blocking', 'requestHeaders' ],
-    callback: onBeforeSendHeadersHandler
-};
-
-vAPI.net.onHeadersReceived = {
-    urls: [ 'http://*/*', 'https://*/*' ],
-    types: [ 'main_frame', 'sub_frame' ],
-    extra: [ 'blocking', 'responseHeaders' ],
-    callback: onHeadersReceived
-};
-
 /*******************************************************************************
 
  Use a `http-equiv` `meta` tag to enforce CSP directives for documents
@@ -614,15 +603,58 @@ vAPI.net.onHeadersReceived = {
 
 /******************************************************************************/
 
-var start = function() {
-    vAPI.net.registerListeners();
-};
+const start = (function() {
+    if (
+        vAPI.net.onBeforeReady instanceof Object &&
+        (
+            vAPI.net.onBeforeReady.experimental !== true ||
+            µMatrix.rawSettings.suspendTabsUntilReady
+        )
+    ) {
+        vAPI.net.onBeforeReady.start();
+    }
 
-/******************************************************************************/
+    return function() {
+        vAPI.net.addListener(
+            'onBeforeRequest',
+            onBeforeRequestHandler,
+            { },
+            [ 'blocking' ]
+        );
 
-return {
-    start: start
-};
+        // https://github.com/uBlockOrigin/uMatrix-issues/issues/74#issuecomment-450687707
+        // https://groups.google.com/a/chromium.org/forum/#!topic/chromium-extensions/vYIaeezZwfQ
+        //   Chromium 72+: use `extraHeaders` to keep the ability to access
+        //   the `Cookie`, `Referer` headers.
+        const beforeSendHeadersExtra = [ 'blocking', 'requestHeaders' ];
+        const wrObsho = browser.webRequest.OnBeforeSendHeadersOptions;
+        if (
+            wrObsho instanceof Object &&
+            wrObsho.hasOwnProperty('EXTRA_HEADERS')
+        ) {
+            beforeSendHeadersExtra.push(wrObsho.EXTRA_HEADERS);
+        }
+        vAPI.net.addListener(
+            'onBeforeSendHeaders',
+            onBeforeSendHeadersHandler,
+            { },
+            beforeSendHeadersExtra
+        );
+
+        vAPI.net.addListener(
+            'onHeadersReceived',
+            onHeadersReceivedHandler,
+            { types: [ 'main_frame', 'sub_frame' ] },
+            [ 'blocking', 'responseHeaders' ]
+        );
+
+        if ( vAPI.net.onBeforeReady instanceof Object ) {
+            vAPI.net.onBeforeReady.stop(onBeforeRequestHandler);
+        }
+    };
+})();
+
+return { start };
 
 /******************************************************************************/
 
